library(MASS)
library(e1071)

#se simulan dos conjuntos de 1000 datos acordes a dos distribuciones 
#normales multivariantes

n <- 2000
p <- 2


#Parmetros estadsticos de los datos
sigmapos <- matrix(c(0.5,0.4,0.4,0.5), 2)
sigmaneg <- matrix(c(0.5,0.2,0.2,0.5), 2)
meanpos <- c(1,1)
meanneg <- c(2,1)


#Nmero de datos positivos y negativos

npos <- round(n / 2)
nneg <- n - npos


#Generacin de datos
set.seed(1)
xpos <- mvrnorm(npos, meanpos, sigmapos)
xneg <- mvrnorm(nneg, meanneg, sigmaneg)
x <- rbind(xpos, xneg)
y <- matrix(c(rep(1, npos), rep(-1,nneg)))

plot(x, col=ifelse(y>0, 1, 2))

#Construccin del Modelo

#1.- Construccin Training - Validation

ntrain <- round(n * 0.7)
tindex <- sample(n, ntrain)
xtrain <- x[tindex,]
xtest <- x[-tindex,]
ytrain <- y[tindex]
ytest <- y[-tindex]

training <- data.frame(cbind(xtrain, ytrain))
names(training)<-c("x1", "x2", "y")
validation <- data.frame(cbind(xtest, ytest))
names(validation)<-c("x1", "x2", "y")

xytrain <- cbind(xtrain, ytrain)
xtrainpos <- xytrain[which(ytrain ==1),][,c(1,2)]
ytrainpos <- xytrain[which(ytrain ==1),][,c(3)]
xtrainneg<- xytrain[which(ytrain ==-1),][,c(1,2)]
ytrainneg<- xytrain[which(ytrain ==-1),][,c(2)]


media1 = c(mean(xtrainpos[,1]),mean(xtrainpos[,2]))
media2 = c(mean(xtrainneg[,1]),mean(xtrainneg[,2]))

desviacion1 <- cov(xtrainpos)
desviacion2 <- cov(xtrainneg)

prior1 = length(ytrainpos) / (length(ytrainpos) + length(ytrainneg))
prior2 = length(ytrainneg) / (length(ytrainpos) + length(ytrainneg))


#2.- Frmula Fischer modificada

Fischer <- function(x, mu1, mu2, sigma1, sigma2, pr1, pr2){
  
  Num <- sign(-0.5* (x - mu1) %*% solve(sigma1) %*% (x - mu1) +
                0.5* (x - mu2) %*% solve(sigma2) %*% (x - mu2) +
                0.5* log(det(sigma2)/det(sigma1), base = exp(1)) -
                log(pr2/pr1, base = exp(1)))
  
  return(Num)
  
}

FischerPred <- vector("numeric", length(ytest))

for (i in 1:length(ytest))
{
  
  FischerPred[i] <- Fischer(xtest[i,], media1, media2, desviacion1, desviacion2, prior1, prior2)
  
}

#Confusion Matrix
table(`Actual Class` = ytest, `Predicted Class` = FischerPred)
error.rate.FischerPred <- sum(ytest != FischerPred)/length(ytest)
print(paste0("Accuary (Precision): ", 1 - error.rate.FischerPred))

#3.- Mtodo QDA

Modelo_QDA <- qda(y~x1 + x2, data = training, prior =c(0.5, 0.5))
Prediction_QDA <- predict(Modelo_QDA, validation)$class

#Confusion Matrix
table(`Actual Class` = ytest, `Predicted Class` = Prediction_QDA)
error.rate.Prediction_QDA <- sum(ytest != Prediction_QDA)/length(ytest)
print(paste0("Accuary (Precision): ", 1 - error.rate.Prediction_QDA))

#4.- Mtodo LDA

Modelo_LDA <- lda(y~x1 + x2, data = training, prior =c(0.5, 0.5))
Prediction_LDA <- predict(Modelo_LDA, validation)$class

#Confusion Matrix
table(`Actual Class` = ytest, `Predicted Class` = Prediction_LDA)
error.rate.Prediction_LDA <- sum(ytest != Prediction_LDA)/length(ytest)
print(paste0("Accuary (Precision): ", 1 - error.rate.Prediction_LDA))

#5.- Mtodo SVM

Modelo_svm <- svm(xtrain, ytrain, kernel = "linear")
Prediction_svm <- predict(Modelo_svm, xtest, decision.values = TRUE)
Prediction_svm <- ifelse(Prediction_svm < 0, -1, 1)

#Confusion Matrix
table(`Actual Class` = ytest, `Predicted Class` = Prediction_svm)
error.rate.Prediction_svm <- sum(ytest != Prediction_svm)/length(ytest)
print(paste0("Accuary (Precision): ", 1 - error.rate.Prediction_svm))
